@@ -305,8 +305,7 @@ update_kbd_cb (GConfClient         *client,
                         }
 
                         key = g_new0 (Key, 1);
-                        if (egg_accelerator_parse_virtual (tmp, &key->keysym, &key->keycode, &key->state) == FALSE
-                            || key->keycode == 0) {
+                        if (!egg_accelerator_parse_virtual (tmp, &key->keysym, &key->keycodes, &key->state)) {
                                 g_free (tmp);
                                 g_free (key);
                                 break;
@@ -361,17 +360,8 @@ init_kbd (GsdMediaKeysManager *manager)
                 }
 
                 key = g_new0 (Key, 1);
-                if (!egg_accelerator_parse_virtual (tmp, &key->keysym, &key->keycode, &key->state)
-                    || key->keycode == 0) {
+                if (!egg_accelerator_parse_virtual (tmp, &key->keysym, &key->keycodes, &key->state)) {
                         g_debug ("Unable to parse: '%s'", tmp);
-
-                        g_free (tmp);
-                        g_free (key);
-                        continue;
-                }
-                /* avoid grabbing all the keyboard when KeyCode cannot be retrieved */
-                if (key->keycode == AnyKey)  {
-                        g_warning ("The shortcut key \"%s\" cannot be found on the current system, ignoring the binding", tmp);
                         g_free (tmp);
                         g_free (key);
                         continue;
@@ -627,6 +617,7 @@ do_sound_action (GsdMediaKeysManager *manager,
                 int threshold = acme_volume_get_threshold (manager->priv->volume);
                 if (vol_step < threshold)
                         vol_step = threshold;
+                g_debug ("Using volume step of %d", vol_step);
         }
 
         /* FIXME: this is racy */
@@ -681,10 +672,16 @@ static gint
 find_by_time (gconstpointer a,
               gconstpointer b)
 {
-        return ((MediaPlayer *)a)->time != 0 && ((MediaPlayer *)a)->time < ((MediaPlayer *)b)->time;
+        return ((MediaPlayer *)a)->time < ((MediaPlayer *)b)->time;
 }
 
-
+/*
+ * Register a new media player. Most applications will want to call
+ * this with time = GDK_CURRENT_TIME. This way, the last registered
+ * player will receive media events. In some cases, applications
+ * may want to register with a lower priority (usually 1), to grab
+ * events only nobody is interested.
+ */
 gboolean
 gsd_media_keys_manager_grab_media_player_keys (GsdMediaKeysManager *manager,
                                                const char          *application,
@@ -694,12 +691,19 @@ gsd_media_keys_manager_grab_media_player_keys (GsdMediaKeysManager *manager,
         GList       *iter;
         MediaPlayer *media_player;
 
+        if (time == GDK_CURRENT_TIME) {
+                GTimeVal tv;
+
+                g_get_current_time (&tv);
+                time = tv.tv_sec * 1000 + tv.tv_usec / 1000;
+        }
+
         iter = g_list_find_custom (manager->priv->media_players,
                                    application,
                                    find_by_application);
 
         if (iter != NULL) {
-                if (time == 0 || ((MediaPlayer *)iter->data)->time < time) {
+                if (((MediaPlayer *)iter->data)->time < time) {
                         g_free (((MediaPlayer *)iter->data)->application);
                         g_free (iter->data);
                         manager->priv->media_players = g_list_delete_link (manager->priv->media_players, iter);
@@ -914,9 +918,8 @@ acme_filter_events (GdkXEvent           *xevent,
         return GDK_FILTER_CONTINUE;
 }
 
-gboolean
-gsd_media_keys_manager_start (GsdMediaKeysManager *manager,
-                              GError             **error)
+static gboolean
+start_media_keys_idle_cb (GsdMediaKeysManager *manager)
 {
         GSList *l;
 
@@ -932,11 +935,6 @@ gsd_media_keys_manager_start (GsdMediaKeysManager *manager,
         init_screens (manager);
         init_kbd (manager);
 
-        /* initialise Volume handler */
-        gnome_settings_profile_start ("acme_volume_new");
-        manager->priv->volume = acme_volume_new ();
-        gnome_settings_profile_end ("acme_volume_new");
-
         /* Start filtering the events */
         for (l = manager->priv->screens; l != NULL; l = l->next) {
                 gnome_settings_profile_start ("gdk_window_add_filter");
@@ -952,6 +950,29 @@ gsd_media_keys_manager_start (GsdMediaKeysManager *manager,
 
         gnome_settings_profile_end (NULL);
 
+        return FALSE;
+}
+
+gboolean
+gsd_media_keys_manager_start (GsdMediaKeysManager *manager,
+                              GError             **error)
+{
+        gnome_settings_profile_start (NULL);
+
+        /* initialise Volume handler
+         *
+         * We do this one here to force checking gstreamer cache, etc.
+         * The rest (grabbing and setting the keys) can happen in an
+         * idle.
+         */
+        gnome_settings_profile_start ("acme_volume_new");
+        manager->priv->volume = acme_volume_new ();
+        gnome_settings_profile_end ("acme_volume_new");
+
+        g_idle_add ((GSourceFunc) start_media_keys_idle_cb, manager);
+
+        gnome_settings_profile_end (NULL);
+
         return TRUE;
 }
 
@@ -996,8 +1017,11 @@ gsd_media_keys_manager_stop (GsdMediaKeysManager *manager)
         }
 
         for (i = 0; i < HANDLED_KEYS; ++i) {
-                g_free (keys[i].key);
-                keys[i].key = NULL;
+                if (keys[i].key) {
+                        g_free (keys[i].key->keycodes);
+                        g_free (keys[i].key);
+                        keys[i].key = NULL;
+                }
         }
 
         if (priv->volume) {
